#include <avr/io.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include "lcd.h"

void data_dir_in(void) {
	//wszystko jako wyjscia
	DDR(LCD_D7PORT) &= ~(1 << LCD_D7);
	DDR(LCD_D6PORT) &= ~(1 << LCD_D6);
	DDR(LCD_D5PORT) &= ~(1 << LCD_D5);
	DDR(LCD_D4PORT) &= ~(1 << LCD_D4);
}

void data_dir_out(void) {
	//wszystko jako wyjscia
	DDR(LCD_D7PORT) |= (1 << LCD_D7);
	DDR(LCD_D6PORT) |= (1 << LCD_D6);
	DDR(LCD_D5PORT) |= (1 << LCD_D5);
	DDR(LCD_D4PORT) |= (1 << LCD_D4);
}

static inline void lcd_sendHalf(uint8_t data) {
	if (data & (1 << 0))
		PORT(LCD_D4PORT) |= (1 << LCD_D4); //static nie udostepnia funkcji do innych moduw
	else
		PORT(LCD_D4PORT) &= ~(1 << LCD_D4); //inline wstawia pelny kod funkcji w kazde miejsce jej wystapienia

	if (data & (1 << 1))
		PORT(LCD_D5PORT) |= (1 << LCD_D5);
	else
		PORT(LCD_D5PORT) &= ~(1 << LCD_D5);

	if (data & (1 << 2))
		PORT(LCD_D6PORT) |= (1 << LCD_D6);
	else
		PORT(LCD_D6PORT) &= ~(1 << LCD_D6);

	if (data & (1 << 3))
		PORT(LCD_D7PORT) |= (1 << LCD_D7);
	else
		PORT(LCD_D7PORT) &= ~(1 << LCD_D7);
}

void _lcd_write_byte(unsigned char _data) {
	//data_dir_out();

	SET_E;
	lcd_sendHalf(_data >> 4);
	CLR_E;
	SET_E;
	lcd_sendHalf(_data);
	CLR_E;

	_delay_us(120);

}

void lcd_write_cmd(uint8_t cmd) {
	CLR_RS;
	_lcd_write_byte(cmd);
}

void lcd_write_data(uint8_t data) {
	SET_RS;
	_lcd_write_byte(data);
}

void lcd_init(void) {
	// inicjowanie pinw portw ustalonych do podczenia z wywietlaczem LCD
	// ustawienie wszystkich jako wyjcia
	data_dir_out();
	DDR(LCD_RSPORT) |= (1 << LCD_RS);
	DDR(LCD_EPORT) |= (1 << LCD_E);

	//_delay_ms(15);
	PORT(LCD_EPORT) &= ~(1 << LCD_E);
	PORT(LCD_RSPORT) &= ~(1 << LCD_RS);

	// tryb 4-bitowy, 2 wiersze, znak 5x7
	lcd_write_cmd( LCDC_FUNC | LCDC_FUNC4B | LCDC_FUNC2L | LCDC_FUNC5x7);
	// wyczenie kursora
	lcd_write_cmd( LCDC_ONOFF | LCDC_CURSOROFF);
	// wczenie wywietlacza
	lcd_write_cmd( LCDC_ONOFF | LCDC_DISPLAYON);
	// przesuwanie kursora w prawo bez przesuwania zawartoci ekranu
	lcd_write_cmd( LCDC_ENTRY | LCDC_ENTRYR);

	// kasowanie ekranu
	lcd_cls();
}

void lcd_cls(void) {
	lcd_write_cmd(LCDC_CLS);
	_delay_ms(4.9);

}

void lcd_str(char * str) {
	register char znak;
	while ((znak = *(str++)))
		lcd_write_data((znak >= 0x80 && znak <= 0x87) ? (znak & 0x07) : znak); //jesli znak zawiera sie w przedziale to maskujemy jego najstarsze bity 0x80 => 0x00
}

#if USE_LCD_CURSOR_HOME == 1
//powracanie kursora na poczatek
void lcd_home(void) {
	lcd_write_cmd(LCDC_CLS | LCDC_HOME);

	_delay_ms(4.9);
}
#endif

#if USE_LCD_CURSOR_ON == 1
//wlaczenie kursora
void lcd_cursor_on(void) {
	lcd_write_cmd( LCDC_ONOFF | LCDC_DISPLAYON | LCDC_CURSORON);
}
// wylaczenie kursora
void lcd_cursor_off(void) {
	lcd_write_cmd( LCDC_ONOFF | LCDC_DISPLAYON);
}
#endif

#if USE_LCD_CURSOR_BLINK == 1
//miganie kursora
void lcd_blink_on(void) {
	lcd_write_cmd( LCDC_ONOFF | LCDC_DISPLAYON | LCDC_CURSORON | LCDC_BLINKON);
}
//nie miganie
void lcd_blink_off(void) {
	lcd_write_cmd( LCDC_ONOFF | LCDC_DISPLAYON);
}
#endif

#if USE_LCD_STR_P == 1
//		 Wysanie stringa do wywietlacza LCD z pamici FLASH
void lcd_str_P(char * str)
{
	register char znak;
	while ( (znak=pgm_read_byte(str++)) )
	lcd_write_data( ( (znak>=0x80) && (znak<=0x87) ) ? (znak & 0x07) : znak);
}
#endif

#if USE_LCD_INT == 1
//		 Wywietla liczb dziesitn na wywietlaczu LCD
void lcd_int(int val) {
	char bufor[17];
	lcd_str(itoa(val, bufor, 10));
}
#endif

#if USE_LCD_LOCATE == 1
//		Ustawienie kursora w pozycji Y-wiersz, X-kolumna
// 		Y = od 0 do 1
// 		X = od 0 do 16
//		funkcja dostosowuje automatycznie adresy DDRAM
//		w zalenoci od rodzaju wywietlacza (ile posiada wierszy)
void lcd_locate(uint8_t y, uint8_t x) {
	switch (y) {
	case 0:
		y = LCD_LINE1;
		break;
	case 1:
		y = LCD_LINE2;
		break; // adres 1 znaku 2 wiersza
	}
	lcd_write_cmd((0x80 + y + x));
}
#endif



